/* verify.c - Verify signatures
 *        Copyright (C) 2001, 2002 Timo Schulz 
 *
 * This file is part of OpenCDK.
 *
 * OpenCDK is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version. 
 *  
 * OpenCDK is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details. 
 *  
 * You should have received a copy of the GNU General Public License 
 * along with OpenCDK; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "opencdk.h"
#include "main.h"
#include "filters.h"
#include "packet.h"


#define GCRY_MD_MD2 5 /* dummy to detect it */

struct
{
  const char *name;
  int algo;
} digest_table[] =
{
  {"MD5",       GCRY_MD_MD5},
  {"SHA1",      GCRY_MD_SHA1},
  {"RIPEMD160", GCRY_MD_RMD160},
  {"MD2",       GCRY_MD_MD2},
  {"TIGER192",  GCRY_MD_TIGER},
  {NULL, 0}
};


static int file_verify_clearsign (CDK_HD hd, const char *, const char *);


int
cdk_file_verify (CDK_HD hd, const char * file, const char * output)
{
  CDK_STREAM inp;
  int rc;

  if (!hd || !file)
    return CDK_Inv_Value;
  
  rc = cdk_stream_open (file, &inp);
  if (rc)
    return rc;
  if (cdk_armor_filter_use (inp))
    {
      /* fixme: check for one-pass sigs? */
      rc = file_verify_clearsign (hd, file, output);
      if (!rc)
        {
          cdk_stream_close (inp);
          return rc;
        }
    }
  rc = _cdk_proc_signature_packets (hd, NULL, inp, NULL);
  cdk_stream_close (inp);
  
  return rc;
}


void
_cdk_result_verify_free (Verify_Result res)
{
  if (res)
    {
      cdk_free (res->notation);
      cdk_free (res);
    }
}


Verify_Result
_cdk_result_verify_new (void)
{
  Verify_Result res;

  res = cdk_calloc (1, sizeof *res);
  if (!res)
    return NULL;
  return res;
}


u32
cdk_sig_get_ulong_attr (CDK_HD hd, int idx, int what)
{
  Verify_Result res;
  u32 val = 0;
  
  if (!hd || !hd->result.verify)
    return 0;

  assert (idx == 0);
  res = hd->result.verify;
  switch (what)
    {
    case CDK_ATTR_CREATED: val = res->created; break;
    case CDK_ATTR_EXPIRE : val = res->expires; break;
    case CDK_ATTR_KEYID  : val = res->keyid[1]; break;
    case CDK_ATTR_STATUS : val = res->sig_status; break;
    case CDK_ATTR_ALGO_PK: val = res->pubkey_algo; break;
    case CDK_ATTR_ALGO_MD: val = res->digest_algo; break;
    default              : val = 0; break;
    }
  
  return val;
}


const void *
cdk_sig_get_data_attr (CDK_HD hd, int idx, int what)
{
  Verify_Result res;
  const void * val;
  
  if (!hd || !hd->result.verify)
    return NULL;
  
  assert (idx == 0);
  res = hd->result.verify;
  switch (what)
    {
    case CDK_ATTR_KEYID   : val = res->keyid; break;
    case CDK_ATTR_NOTATION: val = res->notation; break;
    default               : val = NULL;
    }

  return val;
}


static int
stream_peek (CDK_STREAM inp, char *s, size_t count)
{
  unsigned off;
  int nbytes;
  
  off = cdk_stream_tell (inp);
  nbytes = _cdk_stream_gets (inp, s, count);
  cdk_stream_seek (inp, off);

  return nbytes;
}


static int
file_verify_clearsign (CDK_HD hd, const char * file, const char * output)
{
  CDK_STREAM inp = NULL, out = NULL, tmp = NULL;
  GCRY_MD_HD md = NULL;
  char buf[512], chk[512];
  const char *s;
  int rc = 0;
  int i, is_signed = 0;
  int digest_algo = 0;
  int nbytes;

  if (output)
    {
      rc = cdk_stream_new (output, &out);
      if (rc)
        return rc;
    }
  
  rc = cdk_stream_open (file, &inp);
  if (rc)
    return rc;  

  while (!cdk_stream_eof (inp))
    {
      nbytes = _cdk_stream_gets (inp, buf, sizeof buf-1);
      if (!nbytes)
        break;
      if (!strncmp (buf, "-----BEGIN PGP SIGNED MESSAGE-----", 35))
        {
          is_signed = 1;
          break;
        }
    }
  if (cdk_stream_eof (inp) && !is_signed)
    {
      rc = CDK_Armor_Error;
      goto leave;
    }
  
  while (!cdk_stream_eof (inp))
    {
      nbytes = _cdk_stream_gets (inp, buf, sizeof buf-1);
      if (!nbytes)
        break;
      if (nbytes == 1) /* empty line */
        break;
      else if (!strncmp (buf, "Hash: ", 6))
        {
          for (i = 0; (s = digest_table[i].name); i++)
            {
              if (!strcmp (buf + 6, s))
                {
                  digest_algo = digest_table[i].algo;
                  break;
                }
            }
        }
    }

  if (digest_algo == GCRY_MD_MD2)
    {
      rc = CDK_Inv_Algo;
      goto leave;
    }
  if (!digest_algo)
    digest_algo = GCRY_MD_MD5;
  md = gcry_md_open (digest_algo, 0);
  if (!md)
    {
      rc = CDK_Gcry_Error;
      goto leave;
    }

  s = "-----BEGIN PGP SIGNATURE-----";
  while (!cdk_stream_eof (inp))
    {
      nbytes = _cdk_stream_gets (inp, buf, sizeof buf-1);
      if (!nbytes)
        break;
      if (!strncmp (buf, s, strlen (s)))
        break;
      else
        {
          stream_peek (inp, chk, sizeof chk-1);
          i = strncmp (chk, s, strlen (s));
          if (strlen (buf) == 0 && i == 0)
            continue; /* skip last '\n' */

          _cdk_trim_string (buf, i == 0? 0 : 1);
          gcry_md_write (md, buf, strlen (buf));
        }
      if (!strncmp (buf, "- ", 2))
        memmove (buf, buf + 2, nbytes - 2);
      if (out)
        {
          buf[strlen (buf) - 1] = 0;
          buf[strlen (buf) - 1] = '\n';
          cdk_stream_write (out, buf, strlen (buf));
        }
    }

  tmp = cdk_stream_tmp ();
  if (!tmp)
    {
      rc = CDK_Out_Of_Core;
      goto leave;
    }
  
  s = "-----BEGIN PGP SIGNATURE-----\n";
  cdk_stream_write (tmp, s, strlen (s));
  while (!cdk_stream_eof (inp))
    {
      nbytes = _cdk_stream_gets (inp, buf, sizeof buf-1);
      if (!nbytes)
        break;
      if (nbytes < (sizeof buf -3))
        {
          buf[nbytes-1] = '\n';
          buf[nbytes] = '\0';
        }
      cdk_stream_write (tmp, buf, nbytes);
    }
  cdk_stream_tmp_set_mode (tmp, 0); /* switch to read-mode */
  cdk_stream_seek (tmp, 0);
  cdk_stream_control (tmp, CDK_STREAMCTL_RESET, 1);
  cdk_stream_set_armor_flag (tmp, 0);
  cdk_stream_read (tmp, NULL, 0);

  rc = _cdk_proc_signature_packets (hd, md, tmp, NULL);

 leave:
  cdk_stream_close (out);
  cdk_stream_close (tmp);
  cdk_stream_close (inp);
  return rc;
}
